/*******************************************************************************
 main.c

 Data prijata z desky jsou zaslana do pinheaderu JP10 a vypsana na LED displej FITkitu.

 Copyright (C) 2011 Brno University of Technology,
 Faculty of Information Technology
 Author(s): Lucie Matusova <xmatus21 AT stud.fit.vutbr.cz>

 *******************************************************************************/

#include <fitkitlib.h> 
#include <keyboard/keyboard.h> 
#include <lcd/display.h>

#define BASE_ADDR 0x80  // adresa, na kt. se poslou data
char char_cnt;          // pocet vypsanych znaku na displeji FITkitu
unsigned int connected; // spojeno s pc?
unsigned char value[] = "00";

/*******************************************************************************
 * Vypis uzivatelske napovedy (funkce se vola pri vykonavani prikazu "help")
 *******************************************************************************/
void print_user_help(void) {
    term_send_str_crlf("FPGA->FITkit->PC");
    term_send_str_crlf("FITkit must be properly connected with PC to transmit the data (command IN).");
}

/*******************************************************************************
 * Dekodovani a vykonani uzivatelskych prikazu
 *******************************************************************************/
unsigned char decode_user_cmd(char * cmd_ucase, char * cmd) {
    // setting connection with pc
    // setting connection with pc

    if (cmd_ucase[0] == 'I' && cmd_ucase[1] == 'N') {
        // send confirmation
        term_send_str_crlf("YES");
        // display the confirmation
        if (connected == 0) {
            LCD_append_string("OK");
        }
            
        delay_ms(2000);

        // set flag
        connected = 1;
        return USER_COMMAND;
    }

    return (CMD_UNKNOWN);

}

/*******************************************************************************
 * Inicializace periferii/komponent po naprogramovani FPGA
 *******************************************************************************/
void fpga_initialized() {
    // zapis na LCD
    LCD_init();
    LCD_clear();
    LCD_append_string("FITkit->PC:");
}

/*******************************************************************************
 * Cteni z SPI
 *******************************************************************************/
void spi_read() {
    unsigned char read_addr;
    unsigned int i;
    unsigned char data;

    // nastaveni adresy
    read_addr = BASE_ADDR + 1;

    for (i = 0; i < 2; i++) {
        // precteni dat z SPI
        FPGA_SPI_RW_A8_DN(SPI_FPGA_ENABLE_READ, read_addr, &data, 1);

        // pokud je hodnota na dane adrese rozdilna od minule hodnoty, 
        // vypis ji a uloz do value pro pristi porovnani s dalsi hodnotou
        if (value[i] != data) {

            term_send_hex(read_addr);
            value[i] = data;
            term_send_hex(data);
            
            /*
            if (char_cnt >= 16) {
                LCD_clear();
                char_cnt = 0;
            }
            char_cnt += 1;
            LCD_append_char(data);
            */
        }

        // zvyseni adresy
        read_addr += 1;
    }
}

/*******************************************************************************
 * Hlavni funkce
 *******************************************************************************/
int main(void) {
    unsigned int cnt = 0;
    connected = 0;
    char_cnt = 12;

    initialize_hardware();
    keyboard_init();

    set_led_d6(1); // rozsviceni D6
    set_led_d5(1); // rozsviceni D5
        
    while (1) {
        delay_ms(10);
        cnt++;
        if (cnt > 10) {
            cnt = 0;
            flip_led_d6(); // negace portu na ktere je LED
        }

        terminal_idle(); // obsluha terminalu
        if (connected == 1) {
            spi_read(); // cteni prichozich dat z SPI
        }

    }
}
